import { world, system} from "@minecraft/server";       
import {money_scoreboard_name,open_shop_item,buy_shop,sell_shop} from "./config.js"     //导入配置文件
import { ActionFormData,  ModalFormData,MessageFormData} from '@minecraft/server-ui';

world.afterEvents.worldInitialize.subscribe(() => {
    //用于初始化积分项,如果该世界没有该积分项,则需要在世界生成时添加积分项
    if (world.scoreboard.getObjective(money_scoreboard_name) == null) {
        world.scoreboard.addObjective(money_scoreboard_name, "金币");
    }
});

//玩家使用物品事件监听,类似的事件有很多,手机版玩家可能需要用物品长按才能触发
world.afterEvents.itemUse.subscribe((event)=>{
    let player = event.source   //获取使用物品的玩家
    let item = event.itemStack.typeId   //获取使用的物品

    if(item == `${open_shop_item}`){      
        const form = new ActionFormData()       //创建一个多按钮表单
        form.title("系统商店")      //设置表单标题
        form.button("购买商店")     //添加一个按钮,按钮有两个属性,第一个为名字,后一个为图片路径,图片路径可以为空
        form.button("出售商店")
        form.show(player).then((res)=>{     //使用then来等待该异步表单的返回
            if(res.canceled == false && res.selection != undefined){    //返回的数据中,canceled用于判断是否关闭该表单,若关闭该表单,则selection为未定义
                let index = res.selection
                switch (index) {
                    case 0:
                        show_buy_shop(player)
                        break;
                
                    case 1:
                        show_sell_shop(player)
                        break;
                }
            }
        })
    }
})

function show_sell_shop(player){
    const form = new ActionFormData()   
    form.title("出售商店")
    if(sell_shop.length != 0){
        //遍历增添商品类目按钮
        for(let item of sell_shop){
            form.button(item.show_name , item.icon) //添加一个按钮,按钮有两个属性,第一个为名字,后一个为图片路径,图片路径可以为空
        }
    }
    form.show(player).then((res)=>{
        if(res.canceled == false && res.selection != undefined){
            let index = res.selection
            const _num_form = new ModalFormData()       //创建一个模态表单,表单可以用于用户输出多样的信息,包括滑块,输入框,下拉菜单等
            _num_form.title("请输入出售数量")
            _num_form.textField("出售数量","(整数)")    //创建输入框,第一个参数为输入框名字,第二个参数为输入框提示,第三个为默认值等
            _num_form.show(player).then((res)=>{        //返回的数据在formValues中,这是一个数组,从0开始,每个元素对应一个输入模式的数据
                if(res.canceled == false){
                    let sell_num = parseInt(Number(res.formValues[0]))
                    if(sell_num != NaN &&sell_num>=1){
                        const _confirm_form = new MessageFormData()     //创建一个对话表单,只有两个按钮,button1和button2,用于确认或者弹出消息
                        _confirm_form.title("确认出售")
                        _confirm_form.body(`您确定要出售${sell_shop[index].show_name}共${sell_num}个吗?`)
                        _confirm_form.button1("确定")
                        _confirm_form.button2("取消")
                        _confirm_form.show(player).then((res)=>{
                            if(res.canceled == false && res.selection == 0){
                                sell_item(player,sell_shop[index].price,sell_shop[index].type_id,sell_num)
                            }else{
                                player.sendMessage("出售已取消")
                            }
                        })
                    }else{
                        player.sendMessage("请输入正确的出售数量")
                    }
                    
                }
            })
        }
    })

}

function show_buy_shop(player){
    const form = new ActionFormData()
    form.title("购买商店")
    if(buy_shop.length != 0){
        for(let item of buy_shop){
            form.button(item.show_name , item.icon)
        }
    }
    form.show(player).then((res)=>{
        if(res.canceled == false && res.selection != undefined){
            let index = res.selection
            const _num_form = new ModalFormData()
            _num_form.title("请输入购买数量")
            _num_form.textField("购买数量","(整数)")
            _num_form.show(player).then((res)=>{

                if(res.canceled == false){
                    let buy_num = parseInt(Number(res.formValues[0]))
                    if(buy_num != NaN &&buy_num>=1){
                        const _confirm_form = new MessageFormData()
                        _confirm_form.title("确认购买")
                        _confirm_form.body(`您确定要购买${buy_shop[index].show_name}共${buy_num}个吗?`)
                        _confirm_form.button1("确定")
                        _confirm_form.button2("取消")
                        _confirm_form.show(player).then((res)=>{
                            if(res.canceled == false && res.selection == 0){
                                buy_item(player,buy_shop[index].price,buy_shop[index].type_id,buy_num)
                            }else{
                                player.sendMessage("购买已取消")
                            }
                        })
                    }else{
                        player.sendMessage("请输入正确的购买数量")
                    }
                    
                }
            })
        }
    })
}

function buy_item(player,item_price,item_id,buy_num){
    let player_money = 0
    try {
        //这里需要注意的是,世界的计分板只有玩家参与该计分项目才能被获取,否则会抛出错误
        player_money = world.scoreboard.getObjective(money_scoreboard_name).getScore(player)     
    } catch (error) {
        //当抛出错误后表明该玩家没有参与该积分项目,或其它原因导致无法获取该积分项目
        console.log("该玩家未注册该积分项目")
        world.scoreboard.getObjective(money_scoreboard_name).setScore(player,0)
    }
    let total_price = item_price * buy_num
    if(player_money >= total_price){
        world.scoreboard.getObjective(`${money_scoreboard_name}`).setScore(player,player_money - total_price)
        try {
            //执行命令有多种方法,但无法直接在world类里执行,需要获取维度,所以在world里获取维度后才有运行命令的方法.
            //另一种是获取到玩家对象后,玩家对象也有执行命令的方法
            world.getDimension('overworld').runCommand(`give \"${player.name}\" ${item_id} ${buy_num}`)
        } catch (error) {
            console.log("命令执行错误")
            console.log(error)
        }
        player.sendMessage(`购买成功,花费${total_price}元`)
        player.sendMessage(`您的余额为${world.scoreboard.getObjective(money_scoreboard_name).getScore(player)}`)
    }else{
        player.sendMessage("您的余额不足")
    }
}

function sell_item(player,item_price,item_id,sell_num){
    let player_money = 0
    try {
        player_money = world.scoreboard.getObjective(money_scoreboard_name).getScore(player)     
    } catch (error) {
        console.log("该玩家未注册该积分项目")
        world.scoreboard.getObjective(money_scoreboard_name).setScore(player,0)
    }
    let total_price = item_price * sell_num
    //获取实体组件,任何实体都可以获取一系列组件并修改或查看属性
    //官方文档并未罗列所有实体能获取的实体组件列表
    //需要通过getComponents()方法来获取可以获取的组件列表
    //这里通过获取实体物品栏组件来判断玩家是否有足够库存
    let slot = player.getComponent("minecraft:inventory").container;    //直接获取玩家的物品栏容器对象,该对象有的属性和方法非常多,可以实现背包整理等功能
    let max_slot = slot.size;   //获取背包大小
    let have_conut = 0;
    for (let i = 0; i < max_slot; i++) {
        let item = slot.getItem(i);     //遍历背包每个物品栏的物品信息

        if (item != undefined && item.typeId == item_id) {  //如果某个物品栏里没有物品,为undefined,否则返回物品对象
            have_conut = have_conut + item.amount;
        }
    }
    if (have_conut >= sell_num) {
        try {
            //尝试执行命令.这里需要注意,执行命令时,考虑部分玩家名有空格,需要用双引号包裹,同时双引号需要用转义字符转义
            world.getDimension('overworld').runCommand(`clear \"${player.name}\" ${item_id} 0 ${sell_num}`)
        } catch (error) {
            console.log("命令执行错误")
            console.log(error)
        }
        world.scoreboard.getObjective(`${money_scoreboard_name}`).setScore(player,player_money + total_price)
        player.sendMessage(`出售成功,获得${total_price}元`)
        player.sendMessage(`您的余额为${world.scoreboard.getObjective(money_scoreboard_name).getScore(player)}`)
    }else{
        player.sendMessage("您的库存不足")
    }
}